<?php
/*
 * Created on Sep 17, 2011
 *
 * Author: Filip Bartmann
 * Project: com_ch_pay_j15
 */

defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class plgCHPGoPay extends JPlugin {
	function __construct() {
		JFactory::getLanguage()->load('plg_chp_gopay',JPATH_ADMINISTRATOR);
		$plugin=JPluginHelper::getPlugin('chp','gopay');

		$this->_params=new JParameter($plugin->params);
	}
	
	function getImage() {
		$db=JFactory::getDBO();
		
		$query="SELECT id FROM #__plugins \n";
		$query.="WHERE folder=".$db->quote('chp')." AND element=".$db->quote('gopay');
		$db->setQuery($query);
		$id=$db->loadResult();
		
		$methods=$this->getGateways();
		$gateways=explode(",",$this->_params->get('gateways'));
		
		$options=array();
		for($i=0;$i<count($methods);$i++) {
			if(!in_array($methods[$i]->code,$gateways)) {
				continue;
			}
			$plugin=new stdClass();
			$plugin->id=$id;
			$plugin->value=$methods[$i]->code;
			$plugin->title=JHTML::_('image',$methods[$i]->logo,$methods[$i]->paymentMethod);
			
			$options[]=$plugin;	
		}
		return $options;
	}
	
	private function _getNativeCurrencies() {
		return array('CZK');
	}
	
	private function _getRate($data) {
		$db=JFactory::getDBO();
		$native_currencies=$this->_getNativeCurrencies();
		$rate=1;
		if(!in_array($data->currency,$native_currencies)) {
			$query="SELECT conversion_rate \n";
			$query.="FROM #__chp_courses \n";
			$query.="WHERE from_currency=".$db->quote($data->currency)." AND to_currency=".$db->quote($this->_params->get('main_currency','CZK'));
			$db->setQuery($query);
			$rate=$db->loadResult();
		}
		return $rate;
	}
	
	function getGateways() {
		if($this->_params->get('test_mode')==1) {
			$url='https://testgw.gopay.cz/zaplatit-plna-integrace?';
			$soap_url='https://testgw.gopay.cz/axis/EPaymentService?wsdl';
		} else {
			$url='https://gate.gopay.cz/zaplatit-plna-integrace?';
			$soap_url='https://gate.gopay.cz/axis/EPaymentService?wsdl';
		}
		
		$soap=new SoapClient($soap_url);
		$methods=$soap->paymentMethodList();
		
		return $methods;
	}
	
	function getInfo() {
		$db=JFactory::getDBO();
		$details_page=false;
		$currencies=array();
		foreach($this->_getNativeCurrencies() as $currency) {
			$currencies[]=$db->quote($currency);
		}

		$query="SELECT DISTINCT from_currency \n";
		$query.="FROM #__chp_courses \n";
		$query.="WHERE to_currency IN(".implode(",",$currencies).") AND from_currency NOT IN(".implode(",",$currencies).") \n";
		$db->setQuery($query);
		$currencies=$db->loadResultArray();
		$currencies=array_merge($this->_getNativeCurrencies(),$currencies);
		return array('details_page'=>$details_page,'currencies'=>array_unique($currencies),'check_invoicenum'=>0);
	}
	
	
	private function _hash($data) {
		$data["secret"]=$this->_params->get('secret');
		return bin2hex(sha1(implode("|",$data),true));
	}
	
	private function _encrypt($data) {
		$signature=$this->_hash($data);
		
		$td=mcrypt_module_open(MCRYPT_3DES,'',MCRYPT_MODE_ECB,'');
		$iv=mcrypt_create_iv(mcrypt_enc_get_iv_size($td),MCRYPT_RAND);
		mcrypt_generic_init($td,substr($this->_params->get('secret'),0,mcrypt_enc_get_key_size($td)),$iv);
		$enc_signature=mcrypt_generic($td,$signature);
		mcrypt_generic_deinit($td);
		mcrypt_module_close($td);
		
		return bin2hex($enc_signature);	
	}
	
	private function _decrypt($signature) {
		$td=mcrypt_module_open(MCRYPT_3DES,'',MCRYPT_MODE_ECB,'');
		$iv=mcrypt_create_iv(mcrypt_enc_get_iv_size($td),MCRYPT_RAND);
		mcrypt_generic_init($td,substr($this->_params->get('secret'),0,mcrypt_enc_get_key_size($td)),$iv);
		$decr_signature=mdecrypt_generic($td,$this->_convert($signature));
		mcrypt_generic_deinit($td);
		mcrypt_module_close($td);
		
		return $decr_signature; 
	}
	
	private function _convert($hexString) {
            $hexLenght = strlen($hexString);
            if ($hexLenght % 2 != 0 || preg_match("/[^\da-fA-F]/",$hexString)) return false;
            $binString = "";
            for ($x = 1; $x <= $hexLenght/2; $x++) {
                    $binString .= chr(hexdec(substr($hexString,2 * $x - 2,2)));
            }

            return $binString;
    }

	
	private function _checkData($data,$status) {
		//$rate=$this->_getRate($data);
		if($status->result!="CALL_COMPLETED") {
			return false;
		}
		if($status->sessionState!="WAITING") {
			return false;
		}
		if($status->variableSymbol!=$data->var_sym) {
			return false;
		}
		if($status->productName!=$data->description) {
			return false;
		}
		if($status->eshopGoId!=$this->_params->get('goid')) {
			return false;
		}
		/*if($status->totalPrice!=round($data->amount*$rate*100)) {
			return false;
		}*/
		return true;
	}
	
	function createPay($data,$is_ssl) {
		$rate=$this->_getRate($data);
		if($this->_params->get('test_mode')==1) {
			$url='https://testgw.gopay.cz/zaplatit-plna-integrace?';
			$soap_url='https://testgw.gopay.cz/axis/EPaymentService?wsdl';
		} else {
			$url='https://gate.gopay.cz/zaplatit-plna-integrace?';
			$soap_url='https://gate.gopay.cz/axis/EPaymentService?wsdl';
		}
		
		$soap=new SoapClient($soap_url);
		
		$url_params=array();
		$url_params["eshopGoId"]=$this->_params->get('goid');
		$url_params["productName"]=$data->description;
		$url_params["totalPrice"]=round($data->amount*$rate*100);
		$url_params["variableSymbol"]=$data->var_sym;
		$url_params["failedURL"]=($is_ssl?"https://":"http://").JFactory::getUri()->getHost()."/".JRoute::_('index.php?option=com_chpay&controller=payments&task=pay_cancel&id='.$data->plugin_id,false);
		$url_params["successURL"]=($is_ssl?"https://":"http://").JFactory::getUri()->getHost()."/".JRoute::_('index.php?option=com_chpay&controller=payments&task=pay_finish&id='.$data->plugin_id,false);
		$url_params["encryptedSignature"]=urlencode($this->_encrypt($url_params));
		$url_params["paymentChannels"]=JRequest::getString('payment_id',null,'post');
		
		$name=explode(" ",$data->name);
		
		$customer=array();
		$customer["firstName"]=$name[0];
		$customer["lastName"]=$name[1];
		$customer["email"]=JRequest::getString('email');
		/*$customer["city"]="Ostrava";
		$customer["street"]="V Zahradách 10";
		$customer["postalCode"]="708 00";
		$customer["countryCode"]="CZE";
		
		$customer["phoneNumber"]="731 581 317";*/
		$url_params["customerData"]=$customer;
		
		$status=$soap->__soapCall("createCustomerPaymentSession",array('paymentCommand'=>$url_params));
		if($this->_checkData($data,$status)==false) {
			$this->setError(JText::_('Data validation failed'));
			return false;
		}
		
		$result=array();
		$result[]=$status->eshopGoId;
		$result[]=$status->productName;
		$result[]=$status->totalPrice;
		$result[]=$status->variableSymbol;
		$result[]=$status->result;
		$result[]=$status->sessionState;
		$hash=$this->_hash($result);
		$decryptedSignature=$this->_decrypt($status->encryptedSignature);
		
		if(strcmp(trim($hash),trim($decryptedSignature))!=0) {
			$this->setError(JText::_('Signaure validation failed'));
			return false;
		}
		
		$url_params=array();
		$url_params["sessionInfo.eshopGoId"]=$this->_params->get('goid');
		$url_params["sessionInfo.paymentSessionId"]=$status->paymentSessionId;
		$url_params["sessionInfo.encryptedSignature"]=$this->_encrypt($url_params);
		
		$url.=http_build_query($url_params);
		
		setcookie("payment_id_ck",$data->id,time()+1000*60,'/');

		header("Location: ".$url);
		JFactory::getApplication()->close();
		return true;
		
	}
	
	function payCancel(&$table) {
		$table->load(JRequest::getInt('payment_id_ck','','cookie'));
		setcookie("payment_id_ck","",time()-1000*60,'/');
		return true;
	}
	
	function finishPay(&$data) {
		if($this->_params->get('test_mode')==1) {
			$soap_url='https://testgw.gopay.cz/axis/EPaymentService?wsdl';
		} else {
			$soap_url='https://gate.gopay.cz/axis/EPaymentService?wsdl';
		}
		$session_id=JRequest::getInt('paymentSessionId');
		$var_sym=JRequest::getString('variableSymbol');
		$goid=JRequest::getInt('eshopGoId');
		$data->load(JRequest::getInt('payment_id_ck','','cookie'));
		setcookie("payment_id_ck","",time()-1000*60,'/');
		
		if($goid!=$this->_params->get('goid')) {
			$this->setError(Jtext::_('Returned data validation failed'));
			return false;
		}
		
		$hash=$this->_hash(array($goid,$session_id,$var_sym));
		$decryptedSignature=$this->_decrypt(JRequest::getString('encryptedSignature','','',JREQUEST_ALLOWRAW));
		if(trim($hash)!=trim($decryptedSignature)) {			
			$this->setError(JText::_('Returned signature validation failed'));
			return false;
		}
		
		$pdata=array();
		$pdata["eshopGoId"]=$this->_params->get('goid');
		$pdata["paymentSessionId"]=$session_id;
		$pdata["encryptedSignature"]=$this->_encrypt($pdata);
		
		$soap=new SoapClient($soap_url);
		$status=$soap->paymentStatusGW2($pdata);
		/*if($this->_checkData($pdata,$status)) {
			$this->setError(Jtext::_('Returned data validation failed'));
		}*/
		
		$result=array();
		$result[]=$status->eshopGoId;
		$result[]=$status->productName;
		$result[]=$status->totalPrice;
		$result[]=$status->variableSymbol;
		$result[]=$status->result;
		$result[]=$status->sessionState;
		$result[]=$status->paymentChannel;
		$hash=$this->_hash($result);
		$decryptedSignature=$this->_decrypt($status->encryptedSignature);
		
		if(trim($decryptedSignature)!=trim($hash)) {
			$this->setError(JText::_('Returned signature validation failed'));
			return false;
		}
		
		$data->params=array();
		$methods=$this->getGateways();
		for($i=0;$i<count($methods);$i++) {
			if($methods[$i]->code==$status->paymentChannel) {
				$data->params["details"]=$methods[$i]->paymentMethod;
			}
		}
		
		if($status->sessionState=="PAYMENT_DONE") {
			$data->state=2;
			$data->params["session_id"]=$status->paymentSessionId;
		} else if($status->sessionState=="WAITING") {
			if($status->paymentChannel=="cz_rb") {
				$data->state=0;
				$this->setError(JText::_('Not enought money on account'));
				return false;
			} else {
				$data->state=1;
				$data->params["session_id"]=$status->paymentSessionId;
			}
		} else {
			$data->state=0;
		}
		
		if($data->state==2) {
			$this->_sendMail($data,$var_sym);
		}
		return true;
	}
	
	function makePayment($data) {
		if($this->_params->get('test_mode')==1) {
			$soap_url='https://testgw.gopay.cz/axis/EPaymentService?wsdl';
		} else {
			$soap_url='https://gate.gopay.cz/axis/EPaymentService?wsdl';
		}
		
		$pdata=array();
		$pdata["eshopGoId"]=$this->_params->get('goid');
		$pdata["paymentSessionId"]=JRequest::getString('paymentSessionId',$data->params->get('session_id'),'get');
		$pdata["encryptedSignature"]=$this->_encrypt($pdata);
		
		$soap=new SoapClient($soap_url);
		$status=$soap->paymentStatusGW2($pdata);
		if($this->_checkData($pdata,$status)) {
			$this->setError(Jtext::_('Returned data validation failed'));
			$data->error=Jtext::_('Returned data validation failed');
		}
		
		$result=array();
		$result[]=$status->eshopGoId;
		$result[]=$status->productName;
		$result[]=$status->totalPrice;
		$result[]=$status->variableSymbol;
		$result[]=$status->result;
		$result[]=$status->sessionState;
		$result[]=$status->paymentChannel;
		$hash=$this->_hash($result);
		$decryptedSignature=$this->_decrypt($status->encryptedSignature);
		
		if(trim($decryptedSignature)!=trim($hash)) {
			$this->setError(JText::_('Returned signature validation failed'));
			$data->error=JText::_('Returned signature validation failed');
			return false;
		}
		
		if($status->sessionState=="PAYMENT_DONE") {
			$data->state=2;
			//$this->_sendMail($data,$status->variableSymbol);
			return true;
		}
		return false;
	}
	
	private function _sendMail($data,$var_symbol,$msg=null) {
		$mail=JFactory::getMailer();
		$mail->setSender(array(JFactory::getApplication()->getCfg('mailfrom'),JFactory::getApplication()->getCfg('fromname')));
		$mail->addRecipient($this->_params->get('email'));
		$mail->setSubject(JText::_('PAYMENT_ACCEPTED'));

		switch($data->state) {
			case 0:
				$state=JText::_('Unpayed');
				break;
			case 1:
				$state=JText::_('Authorized');
				break;
			case 2:
				$state=JText::_('Payed');
				break;
		}
		$body=JText::_('You recieved payment via GoPay gateway')." \n";
		$body.=JText::sprintf('Invoice num. %s',$var_symbol)." \n";
		$body.=JText::sprintf('Amount: %s %s',$data->amount,$data->currency)." \n";
		$body.=JText::sprintf('Description: %s',$data->description)." \n";
		$body.=JText::sprintf('Username: %s',$data->name)." \n";
		$body.=JText::sprintf('Company: %s',$data->company)." \n";
		$body.=JText::sprintf('Date payment: %s',JFactory::getDate($data->date_payed)->toFormat("%d.%m.%Y %H:%M"))." \n";
		$body.=JText::sprintf('State: %s',$state)." \n";
		if($data->state==0) {
			$body.=JText::sprintf('Error: %s',$msg);
		}
		$mail->setBody($body);
		$res=$mail->Send();
	}
}

?>