<?php
/*
 * Created on 3.5.2011
 *
 * Author: Filip Bartmann
 * Project: com_ch_pay_j15
 */

defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class plgCHPPayMuzo extends JPlugin {
	private $_params=null;
	private $_data=null;
	function __construct() {
		JFactory::getLanguage()->load('plg_chp_paymuzo',JPATH_ADMINISTRATOR);
		$plugin=JPluginHelper::getPlugin('chp','paymuzo');
		$this->_params=new JParameter($plugin->params);
		$this->_data=new stdClass();
	}

	function getImage() {
		return JText::_('PAYMUZO_IMAGE');
	}

	private function _getNativeCurrencies() {
		return array('CZK');
	}

	function getInfo() {
		$db=JFactory::getDBO();
		$details_page=false;
		$currencies=array();
		foreach($this->_getNativeCurrencies() as $currency) {
			$currencies[]=$db->quote($currency);
		}

		$query="SELECT DISTINCT from_currency \n";
		$query.="FROM #__chp_courses \n";
		$query.="WHERE to_currency IN(".implode(",",$currencies).") AND from_currency NOT IN(".implode(",",$currencies).") \n";
		$db->setQuery($query);
		$currencies=$db->loadResultArray();
		$currencies=array_merge($this->_getNativeCurrencies(),$currencies);
		return array('details_page'=>$details_page,'currencies'=>array_unique($currencies),'check_invoicenum'=>1);
	}

	private function _getRate($data) {
		$db=JFactory::getDBO();
		$native_currencies=$this->_getNativeCurrencies();
		$rate=1;
		if(!in_array($data->currency,$native_currencies)) {
			$query="SELECT conversion_rate \n";
			$query.="FROM #__chp_courses \n";
			$query.="WHERE from_currency=".$db->quote($data->currency)." AND to_currency=".$db->quote($this->_params->get('main_currency','CZK'));
			$db->setQuery($query);
			$rate=$db->loadResult();
		}
		return $rate;
	}

	function createPay($data,$is_ssl=true) {
		$rate=$this->_getRate($data);
		if($this->_params->get('test_mode')==1) {
			$url='https://test.3dsecure.gpwebpay.com/kb/order.do?';
		} else {
			$url='https://3dsecure.gpwebpay.com/kb/order.do?';
		}
		
		$db=JFactory::getDBO();
		$query="SELECT COUNT(*) FROM #__chp_payments \n";
		$query.="WHERE plugin_id=(SELECT id FROM #__plugins WHERE folder=".$db->quote('chp')." AND element=".$db->quote('paymuzo').")";
		$db->setQuery($query);
		$cnt=$db->loadResult();
		
		$order_num=1000+$cnt;

		$url_params=array();
		$url_params["MERCHANTNUMBER"]=$this->_params->get('merchant');
		$url_params["OPERATION"]="CREATE_ORDER";
		//$url_params["ORDERNUMBER"]=$data->var_sym;
		$url_params["ORDERNUMBER"]=$order_num;
		$url_params["AMOUNT"]=round($data->amount*$rate*100);
		$url_params["CURRENCY"]=203;
		$url_params["DEPOSITFLAG"]=$this->_params->get('deposit');
		$url_params["URL"]=($is_ssl?"https://":"http://").JFactory::getURI()->getHost().'/plugins/chp/npm.php';
		$description=strtr($data->description,"áäčďéěíĺľňóô öŕšťúů üýřžÁÄČĎÉĚÍĹĽŇÓÔ ÖŔŠŤÚŮ ÜÝŘŽ","aacdeeillnoo orstuu uyrzAACDEEILLNOO ORSTUU UYRZ");
		$url_params["DESCRIPTION"]=$description;
		$url_params["MD"]=$data->id."%".$data->var_sym;
		$digest=$this->_createDigest($url_params);
		$url_params["DIGEST"]=$digest;


		$url.=http_build_query($url_params);

		header("Location: ".$url);
		JFactory::getApplication()->close();
		return true;
	}

	private function _createDigest($digest) {
		return $this->_signDigest(implode("|",array_values($digest)));
	}

	private function _signDigest($data) {
		$sdata="";
		if(strlen($this->_params->get('private_passwd'))) {
			$pkey=openssl_pkey_get_private('file://'.$this->_params->get('private_key'),$this->_params->get('private_passwd'));
		} else {
			$pkey=openssl_pkey_get_private('file://'.$this->_params->get('private_key'));
		}

		openssl_sign($data,$sdata,$pkey);
		openssl_free_key($pkey);
		$sdata=base64_encode($sdata);

		return $sdata;
	}

	private function _verifyDigest($signature,$digest) {
		$pub_key=openssl_pkey_get_public('file://'.$this->_params->get('public_key'));
		$signature=base64_decode($signature);
		$res=openssl_verify($digest,$signature,$pub_key);
		openssl_free_key($pub_key);
		return $res==1?true:false;
	}

	function finishPay(&$data) {
		$r_data=JRequest::getString('data','post',JREQUEST_ALLOWRAW);
		$r_data=base64_decode($r_data);
		$request=array();
		parse_str($r_data,$request);
		
		$md=explode("%",$request["MD"]);

		//$var_symbol=$request["ORDERNUMBER"];
		$var_symbol=$md[1];
		$data->load($md[0]);
		$prcode=$request["PRCODE"];
		$srcode=$request["SRCODE"];
		$msg=$request["RESULTTEXT"];
		$signature=$request["DIGEST"];
		$digest=array("CREATE_ORDER",$request["ORDERNUMBER"],$data->id."%".$data->var_sym,$prcode,$srcode,$msg);
		if(!$this->_verifyDigest($signature,implode("|",$digest))) {
			$this->setError(JText::_('Failed to verify response'));
			return false;
		}
		if($prcode==0 && $srcode==0) {
			if($this->_params->get('deposit')==1) {
				$data->state=2;
			} else {
				$data->state=1;
			}
			$this->_sendMail($data,$var_symbol);
			return true;
		} else {
			$this->setError($this->_checkError($prcode,$srcode,$msg));
			$this->_sendMail($data,$var_symbol,$msg);
			return false;
		}

	}

	private function _sendMail($data,$var_symbol,$msg=null) {
		$mail=JFactory::getMailer();
		$mail->setSender(array(JFactory::getApplication()->getCfg('mailfrom'),JFactory::getApplication()->getCfg('fromname')));
		$mail->addRecipient($this->_params->get('email'));
		$mail->setSubject(JText::_('PAYMENT_ACCEPTED'));

		switch($data->state) {
			case 0:
				$state=JText::_('Unpayed');
				break;
			case 1:
				$state=JText::_('Authorized');
				break;
			case 2:
				$state=JText::_('Payed');
				break;
		}
		$body=JText::_('You recieved payment via PayMuzo gateway')." \n";
		$body.=JText::sprintf('Invoice num. %s',$var_symbol)." \n";
		$body.=JText::sprintf('Amount: %s %s',$data->amount,$data->currency)." \n";
		$body.=JText::sprintf('Description: %s',$data->description)." \n";
		$body.=JText::sprintf('Username: %s',$data->name)." \n";
		$body.=JText::sprintf('Company: %s',$data->company)." \n";
		$body.=JText::sprintf('Date payment: %s',JFactory::getDate($data->date_payed)->toFormat("%d.%m.%Y %H:%M"))." \n";
		$body.=JText::sprintf('State: %s',$state)." \n";
		if($data->state==0) {
			$body.=JText::sprintf('Error: %s',$msg);
		}
		$mail->setBody($body);
		$res=$mail->Send();
	}

	function makePayment($data) {
		$rate=$this->_getRate($data);
		if($this->_params->get('test_mode')==1) {
			$wsdl_file='pgw-standard_test.wsdl';
		} else {
			$wsdl_file='pgw-standard.wsdl';
		}
		$soap=new SoapClient(JPATH_SITE.DS.'plugins'.DS.'chp'.DS.'assets'.DS.$wsdl_file);
		$digest=array($this->_params->get('merchant'),$data->var_sym,intval($data->amount*$rate));
		$res=$soap->deposit($this->_params->get('merchant'),$data->var_sym,intval($data->amount*$rate),$this->_signDigest(implode('|',$digest)));
		$digest=array($res->orderNumber,$res->primaryReturnCode,$res->secondaryReturnCode);
		if(!$this->_verifyDigest($res->digest,implode("|",$digest))) {
			$this->setError(JText::_('Failed to verify response'));
			return false;
		}
		if($res->primaryReturnCode!=0) {
			$this->setError(JText::_('Failed to make payment'));
			return false;
		}
		return true;
	}

	private function _checkError($prcode,$srcode,$msg) {
		switch($prcode) {
			case 1:
				return JText::sprintf('FIELD %S IS TOO LONG',$this->_checkField($srcode));
				break;
			case 2:
				return JText::sprintf('FIELD %S IS TOO SHORT',$this->_checkField($srcode));
				break;
			case 3:
				return JText::sprintf('FIELD %S IS INCORRECT',$this->_checkField($srcode));
				break;
			case 4:
				return Jtext::sprintf('FIELD %S IS EMPTY',$this->_checkField($srcode));
				break;
			case 5:
			case 15:
				return Jtext::sprintf('FIELD %S IS MISSING',$this->_checkField($srcode));
				break;
			default:
				return $msg;
				break;
		}
	}

	private function _checkField($srcode) {
		switch($srcode) {
			case 1:
				return 'ORDERNUMBER';
				break;
			case 2:
				return 'MERCHANTNUMBER';
				break;
			case 6:
				return 'AMOUNT';
				break;
			case 7:
				return 'CURRENCY';
				break;
			case 8:
				return 'DEPOSITFLAG';
				break;
			case 10:
				return 'MERORDERNUM';
				break;
			case 11:
				return 'CREDITNUMBER';
				break;
			case 12:
				return 'OPERATION';
				break;
			case 18:
				return 'BATCH';
				break;
			case 22:
				return 'ORDER';
				break;
			case 24:
				return 'URL';
				break;
			case 25:
				return 'MD';
				break;
			case 26:
				return 'DESC';
				break;
			case 34:
				return 'DIGEST';
				break;
		}
	}
}
?>